<?php
/**
 * SpecialHierarchy.php
 *
 * This file is part of the Hierarchy extension for MediaWiki; supporting the
 * Treeview skin.  Its code performs two main functions: ensuring cache-
 * consistency and providing server-side support for client-side load-on-demand
 * of treeview branches.
 *
 * Licenced under the General Public Licence 2, without warranty.
 *
 * @licence GPL2 http://www.gnu.org/copyleft/gpl.html
 * @author Laird Shaw <lairdshaw77@gmail.com>
 */
define('TV_RELEASE_VERSION', '1.6.8');
# Register extension
$wgExtensionFunctions[] = 'wfSpecialHierarchy';
$wgExtensionCredits['specialpage'][] = array(
	'name' => 'Hierarchy',
	'author' => 'Laird Shaw',
	'version' => TV_RELEASE_VERSION,
	'url' => 'http://creativeandcritical.net/mediawiki_treeview_skin',
	'description' => 'Generates the treeview hierarchy for the Treeview skin.'
);

$dir = dirname( __FILE__ ) . '/';
$wgExtensionMessagesFiles['Hierarchy'] = $dir.'Hierarchy.i18n.php';

use MediaWiki\MediaWikiServices;

global $IP; # not required for MW 1.5.x or 1.6.x
require_once('Hierarchy.php');
/** for FauxRequestEx */
require_once('State.php');
if (file_exists("$IP/includes/SpecialPage.php")) {
	require_once("$IP/includes/SpecialPage.php");
} else	require_once("$IP/includes/specialpage/SpecialPage.php");
/** for wfGetRedirectsTo() and wfGetSeeAlsos() */
require_once('UtilityFunctions.php');
/** for TvQueryString::queryStrToArr() */
require_once('QueryString.php');

$wgHooks['ArticleDeleteComplete'][] = 'wfHierarchy_ArticleDeleteCompleteHook';
$wgHooks['ArticleUndelete'][] = 'wfHierarchy_UndeleteCompleteHook';
$wgHooks['PageContentSave'][] = 'wfHierarchy_PageContentSaveHook';
$wgHooks['PageContentSaveComplete'][] = 'wfHierarchy_PageContentSaveCompleteHook';
$wgHooks['TitleMoveComplete'][] = 'wfHierarchy_TitleMoveCompleteHook';
$wgHooks['UploadComplete'][] = 'wfHierarchy_UploadCompleteHook';
$wgHooks['CategoryAfterPageAdded'][] = 'wfHierarchy_CategoryAfterPageAddedOrRemovedHook';
$wgHooks['CategoryAfterPageRemoved'][] = 'wfHierarchy_CategoryAfterPageAddedOrRemovedHook';
$wgHooks['UnknownAction'][] = 'wfHierarchy_UnknownActionHook';
$wgHooks['SkinTemplateNavigation'][] = 'wfHierarchy_SkinTemplateNavigationHook';
$wgHooks['ArticleViewRedirect'][] = 'wfHierarchy_ArticleViewRedirectHook';

global $wgTV_RedirectedFrom;
$wgTV_RedirectedFrom = false;

/** Adds tabs for the rebuildtreeview, parsetreeview, treeviewcachestatus
  * actions when the current page is the (pseudo-) Treeview/Hierarchy
  * configuration page.
  */
function wfHierarchy_SkinTemplateNavigationHook(&$skin, &$content_navigation) {
	global $wgArticle, $wgTitle, $wgRequest, $wgUser;
	$t = ($wgArticle ? $wgArticle->getTitle() : $wgTitle);
	$action = $wgRequest->getText('action');
	$res = Hierarchy::isConfigPage($t);
	if ($res !== false) {
		if ($wgUser->isAllowed('parsetreeview')) {
			$content_navigation['actions']['parsetreeview'] = array(
				'class' => $action == 'parsetreeview' ?
				            'selected' : 'tvaction',
				'text'  => wfMessage('tv_parsetreeview')
				             ->escaped(),
				'href'  => $t->getLocalUrl(
				            'action=parsetreeview')
			);
		}
		if ($wgUser->isAllowed('rebuildtreeview')) {
			$content_navigation['actions']['rebuildtreeview'] = array(
				'class' => $action == 'rebuildtreeview' ?
				            'selected' : 'tvaction',
				'text'  => wfMessage('tv_rebuildtreeview')
				             ->escaped(),
				'href'  => $t->getLocalUrl(
				            'action=rebuildtreeview')
			);
		}
		if ($wgUser->isAllowed('treeviewcachestatus')) {
			$content_navigation['actions']['treeviewcachestatus'] = array(
				'class' => $action == 'treeviewcachestatus' ?
				            'selected' : 'tvaction',
				'text'  => wfMessage('tv_treeviewcachestatus')
				             ->escaped(),
				'href'  => $t->getLocalUrl(
				            'action=treeviewcachestatus')
			);
		}
	}
	return true;
}

/** This can't be hooked in the SkinTreeview file since that file only gets
  * included at a later point than this function needs to be called.
  */
function wfHierarchy_ArticleViewRedirectHook($article) {
	global $wgTV_RedirectedFrom;
	$sk = RequestContext::getMain()->getSkin();
	$wgTV_RedirectedFrom = $article->mRedirectedFrom;

	return (isset($sk->mDisableDefaultRedirectSubHeadings) &&
	        $sk->mDisableDefaultRedirectSubHeadings)
	         ? false
	         : true;
}

/** Implements the rebuildtreeview, parsetreeview, treeviewcachestatus actions
  * as reached through the (pseudo-) configuration page's corresponding tabs:
  * 'Parse configuration', 'Rebuild treeview', 'Cache status'
  */
function wfHierarchy_UnknownActionHook($action, $article) {
	global $wgOut, $wgUser, $wgHierarchyPage;
	if ($action == 'view_tv') {
		global $wgUseFileCache;
		wfDebug('wfHierarchy_UnknownActionHook: setting '.
		  "$wgUseFileCache false.\n");
		$wgUseFileCache = false;
		$article->view();
		$handled = true;
	} else if ($action == 'rebuildtreeview' || $action == 'parsetreeview' ||
	  $action == 'treeviewcachestatus') {
		$handled = true;
		if (!$wgUser->isAllowed($action)) {
			$handled = false;
			$wgOut->permissionRequired($action);
		} else {
			$res = Hierarchy::isConfigPage($article->getTitle());
			if ($res === false) {
				$sk = $wgUser->getSkin();
				/* This repeats the title creation within
				 * isConfigPage() but it is rarely reached and
				 * is not time-critical.
				 */
				$tConfig = Hierarchy::getConfigPageTitle(true);
				$configHref = $tConfig ?
				  $sk->makeLinkObj($tConfig) :
				  wfMessage('tv_invalid')->plain().wfMessage(
				    'tv_invconfpg', $wgHierarchyPage)->plain();
				$wgOut->addHTML(wfMessage(
				  'tv_err_actionforconf', $action,
				  $configHref)->escaped());
			} else {
				$s = '<h1>'.wfMessage('tv_'.$action)->escaped().
				  "</h1>\n";
				$hObj = new Hierarchy;
				if ($action == 'rebuildtreeview' ||
				    $action == 'treeviewcachestatus') {
					/* This is called prior to the call to
					 * parseConfigPage() so that a re-parse
					 * is avoided. */
					$cacheHit = $hObj->loadHierarchy();
					$ts = $hObj->getHierarchyTs();
				}
				if ($action != 'treeviewcachestatus') {
					$parseRes = $hObj->parseConfigPage();
				}
				if ($res === 0) {
					$s .= '<h2>'.
					  wfMessage('tv_standin_hdr')
					    ->escaped()."</h2>\n".
					  wfMessage('tv_standinconfpg_desc')
					    ->escaped()."\n";
				}
				if ($action == 'rebuildtreeview') {
					/* The parse result is stored in the
					 * object so that this call does not
					 * invoke a re-parse. */
					$hObj->buildHierarchyAndRemappings();
					$hObj->saveRemappings();
					$hObj->saveHierarchy();
					$cacheHit2 = $hObj->loadHierarchy();
					$ts2 = $hObj->getHierarchyTs();
					$k = $cacheHit2 ?
					  'tv_rebuildsucc' : 'tv_rebuildmiss';
					$postStat = '<h2>'.wfMessage($k.'_hdr')
					  ->escaped()."</h2>\n".wfMessage($k)
					  ->escaped().'<h2>'.wfMessage(
					  'tv_cachestatpost_hdr')->escaped().
					  "</h2>\n".wfMessage($cacheHit2 === 0 ?
					   'tv_unusablecachehit' : ($cacheHit2 ?
					    'tv_cachehit':'tv_cachemiss'),$ts2)
					    ->escaped()."\n";
				} else {
					$cacheHit2 = false;
					$postStat = '';
				}
				/* Output the result of the cache retrieval(s).
				 * Warn if the cache seems non-functional. */
				if ($action == 'rebuildtreeview' ||
				    $action == 'treeviewcachestatus') {
					global $wgMainCacheType;
					if ($cacheHit2 === false &&
					  $cacheHit === false &&
					  $wgMainCacheType == CACHE_NONE) {
						$s .= '<h2>'.wfMessage(
						  'tv_nocache_hdr')->escaped().
						  "</h2>\n".wfMessage(
						  'tv_nocache')->escaped();
					}
					$s .= $postStat;
					$hdrKey = $action == 'rebuildtreeview' ?
					  'tv_cachestatpre_hdr' :
					  'tv_cachestatus_hdr';
					$s .= '<h2>'.wfMessage($hdrKey)
					  ->escaped().
					  "</h2>\n".wfMessage($cacheHit === 0 ?
					    'tv_unusablecachehit' : ($cacheHit ?
					     'tv_cachehit':'tv_cachemiss'),$ts)
					     ->escaped()."\n";
				}
				/* Output parse diagnostics. */
				if ($action != 'treeviewcachestatus') {
					$s .= '<h2>'.wfMessage(
					  'tv_parseerrintro')->escaped().
					  "</h2>\n".($parseRes ?
					    wfMessage('tv_noproblems')
					    ->escaped() :
					    $hObj->getParseErrors_Html());
				}
				$s .= '<h1>'.wfMessage('tv_content')->escaped()
				  ."</h1>\n";
				$wgOut->addHTML("$s\n");
				$wgOut->addWikiText($hObj->getConfigPageText());
			}
		}
	} else	$handled = false;
	/* For MediaWiki 1.6.5, a return value of false is interpreted as
	 * 'unhandled'; for MediaWiki 1.5.6, 1.7.1 and 1.12.0, a return value of
	 * true is interpreted in that way.  We cater to the later versions
	 * here.
	 */
	return !$handled;
}

function wfHierarchy_ArticleDeleteCompleteHook() {
	wfDebug('wfHierarchy_ArticleDeleteCompleteHook: Invoking '.
	  "indiscriminate invalidation.\n");
	HierarchyPage::invalidateAll();
	return true;
}

function wfHierarchy_UndeleteCompleteHook() {
	wfDebug('wfHierarchy_UndeleteCompleteHook: Invoking indiscriminate '.
	  "invalidation.\n");
	HierarchyPage::invalidateAll();
	return true;
}

function wfHierarchy_TitleMoveCompleteHook() {
	wfDebug('wfHierarchy_TitleMoveCompleteHook: Invoking indiscriminate '.
	  "invalidation.\n");
	HierarchyPage::invalidateAll();
	return true;
}

/**
 * The one and only non-configuration global used by Treeview-related code: it's
 * used only to work around the parts of MediaWiki's hook system that aren't
 * designed for Treeview purposes.
 */
global $wgSpecialHierarchySavedHookData;
$wgSpecialHierarchySavedHookData = array('invalidatedall' => false);

function wfHierarchy_PageContentSaveHook($wikiPage) {
	global $wgSpecialHierarchySavedHookData;

	$hookData = array();
	$t = $wikiPage->getTitle();
	$id = $t->getArticleID();
	# Trigger on all page creations (this test could be refined) ...
	$invalidate = ($id == 0);
	if ($invalidate) wfDebug(__FUNCTION__.": detected a page creation\n");
	else {
		# ...and on edits to the hierarchy config page...
		if (Hierarchy::isConfigPage($t)) {
			$invalidate = true;
			wfDebug(__FUNCTION__.": detected a save to the config page\n");
		}
	}
	# ... and on edits to any messages upon which that config depends
	if (!$invalidate) {
		foreach (Hierarchy::getMessageDeps() as $msg) {
			# MediaWiki always capitalises the first letter of
			# articles in the message namespace upon save even when
			# $wgCapitalLinks = false
			$tMsg = Title::newFromText(ucfirst($msg), NS_MEDIAWIKI);
			if ($id == $tMsg->getArticleID()) {
				$invalidate = true;
				wfDebug(__FUNCTION__.": detected a saved edit to the ".
				  "message '$msg', upon which the config page ".
				  "depends\n");
				break;
			}
		}
	}
	if ($invalidate) {
		$hookData['invalidatedall'] = true;
	} else {
		# ... and possibly later if this is a changing redirect
		$tTarget = false;
		if (($wikiPage = WikiPage::factory($t))) {
			$c = $wikiPage->getContent();
			$tTarget = $c->getRedirectTarget();
			if ($tTarget) {
				$hookData['old_target'] = $tTarget;
				$a = wfGetRedirectsTo($tTarget);
				$a[] = $tTarget;
				$hookData['old_aliaslist'] = $a;
			}
		}
		if ($tTarget) {
			$hookData = null;
		}
	}
	$wgSpecialHierarchySavedHookData = $hookData;
	return true;
}

function wfHierarchy_PageContentSaveCompleteHook($wikiPage, $user, $content) {
	global $wgSpecialHierarchySavedHookData;
	$hookData = $wgSpecialHierarchySavedHookData;
	$t = $wikiPage->getTitle();
	if (isset($hookData['invalidatedall']) && $hookData['invalidatedall']) {
		HierarchyPage::invalidateAll();
		if (isset($hookData['treeviewtagran'])) {
			# Invalidate the parser cache of the saved article
			# because the treeview tag was built using the
			# invalidated treeview.
			wfDebug(__METHOD__.': invalidating the cache of the '.
			  'saved new page due to <treeview> tag use.');
			$t->invalidateCache();
		}
		return true;
	}
	$tTarget = $content->getRedirectTarget();
	$tOldTarget = $hookData['old_target'];
	# If this was a redirect that post-save no longer redirects to the same
	# page ...
	if ($tOldTarget && (!$tTarget ||
	  $tOldTarget->getArticleID() != $tTarget->getArticleID())) {
		# ...then invalidate the article that it previously redirected
		# to and all other articles that redirected to that article.
		$invalArr = $hookData['old_aliaslist'];
		$titleInList = true;
	} else{
		$invalArr = array();
		$titleInList = false;
	}
	# If this is now a redirect that points to a different page than
	# pre-save ...
	if ($tTarget && (!$tOldTarget ||
	  $tOldTarget->getArticleID() != $tTarget->getArticleID())) {
		# ...then invalidate the article that it now redirects to and
		# all other articles that redirect to that article.
		$a = wfGetRedirectsTo($tTarget);
		if ($titleInList) $a[] = $tTarget;
		$invalArr = array_merge($invalArr, $a);
	}
	if ($invalArr) {
		$dbgList = '';
		foreach ($invalArr as $t) {
			if ($dbgList) $dbgList .= ', ';
			$dbgList .= $t->getPrefixedText();
		}
		wfDebug(__FUNCTION__.": detected a new or changed redirect - ".
		  "INVALIDATING the cache of these articles: $dbgList\n");
		foreach($invalArr as $title) {
			$title->invalidateCache();
		}
	}
	return true; # warning: prior conditional exit
}

/**
 * This is the lazy approach - better would be to check whether the category in
 * question is being added to or removed from the tree as part of a CATEGORY
 * directive, and only invalidate in those cases.
 */
function wfHierarchy_CategoryAfterPageAddedOrRemovedHook() {
	global $wgSpecialHierarchySavedHookData;

	if (isset($wgSpecialHierarchySavedHookData['invalidatedall']) &&
	  $wgSpecialHierarchySavedHookData['invalidatedall']) return true;

	HierarchyPage::invalidateAll();

	return true;
}

/**
 * This is overkill when the image already exists - as long as its url doesn't
 * change there's no need to rebuild things - but if the upload is for a new
 * image then this invalidation is required because the url will have been
 * unknown to the treeview.  An optimisation would be to distinguish between
 * the two.
 */
function wfHierarchy_UploadCompleteHook($img) {
	$method = method_exists($img, 'getName') ? 'getName' : 'getTitle';
	$imageName = $img->$method();
	if (Hierarchy::usesImage($imageName)) {
		wfDebug(__FUNCTION__.": Uploaded image '$imageName' is used by the ".
		  "treeview: invoking indiscriminate invalidation.\n");
		HierarchyPage::invalidateAll();
	}
	return true;
}

/** This class implements a special page used by the Treeview skin to download
  * through javascript only the necessary nodes when expanding an unloaded part
  * of the treeview.
  * @package MediaWiki
  * @subpackage Extensions
  */
class HierarchyPage extends SpecialPage {
	/**#@+ @access public */

	/** Constructor for HierarchyPage. */
	function __construct() {
		parent::__construct('Hierarchy');
	}

	/** Main entry point function for HierarchyPage as a SpecialPage. */
	function execute($par) {
		/* Avoid $wgRequest because it includes cookies. */
		$queryParams = $_GET;
		unset($queryParams['title']);
		if (!$queryParams) {
			$this->showMainControls();
		} else	$this->handleLoadOnDemand_Xml();
	}

	function showMainControls() {
		global $wgOut, $wgUser, $wgHierarchyPage;
		$this->setHeaders();
		$wgOut->addHTML('<h1>Hierarchy/Treeview controls</h1>'."\n");
		list($tConfig, $fullMatch) = Hierarchy::getConfigPageTitle_();
		if (!$tConfig) {
			$wgOut->addHTML(wfMessage('tv_invconfpg',
			  $wgHierarchyPage)->escaped()."\n");
		} else {
			$sk = $this->getSkin();
			$confPage = $fullMatch? 'confpg': 'standinconfpg';
			$confTitle = $tConfig->getPrefixedText();
			$wgOut->addHTML('<dl>'."\n");
			foreach (array(
			  $confPage             => false,
			  'parsetreeview'       => true,
			  'rebuildtreeview'     => true,
			  'treeviewcachestatus' => true)
			as $key => $isAction) {
				if ($isAction && !$wgUser->isAllowed($key)) {
					continue;
				}
				$desc = wfMessage('tv_'.$key.'_desc')
				  ->escaped();
				$linkFunc = $isAction ?
				  'makeKnownLink' : 'makeLink';
				$linkRenderer = MediaWikiServices::getInstance()
				  ->getLinkRenderer();
				$link = $linkRenderer->$linkFunc($tConfig,
				  wfMessage('tv_'.$key, $confTitle)->escaped(),
				  [], $isAction ? ['action' => $key] : []);
				$wgOut->addHTML("<dt>$link</dt>\n".
				  "<dd>$desc</dd>\n");
			}
			$wgOut->addHTML('</dl>'."\n");
		}
	}

	/** Handles a load-on-demand request for a (set of) nodes.  Interprets
	  * the url query parameters, builds the branch(es) and outputs XML in
	  * place of MediaWiki's default output mechanisms.  The request is
	  * issued, and its result interpreted, by code in
	  * [mediawikiroot]/skins/treeview/treeview.js
	  */
	function handleLoadOnDemand_Xml() {
		global $wgOut, $wgInputEncoding, $wgRequest, $wgUser;
		/* Avoid E_NOTICE warnings during accesses to undefined array
		 * keys.*/
		wfSuppressWarnings();

		$nodeIds = explode(',', $wgRequest->getText('expnodeids'));
		$base = $wgRequest->getVal('base');
		/* WebRequest::normalizeUnicode performs (all/most of?) the
		 * equivalent of urldecode. */
		$effFullUrl = /*urldecode(*/$wgRequest->getVal('effurl')/*)*/;
		$tmp = strpos($effFullUrl, '?');
		$effQueryStr = $tmp!==false ? substr($effFullUrl, $tmp+1) : '';
		$fauxParams = TvQueryString::queryStrToArr($effQueryStr, false);
		if ($base && preg_match("#$base/(.*)#",$effFullUrl,$effTitle)) {
			$effTitle = $effTitle[1];
		} else {
			$effTitle = $wgRequest->getVal('effttl');
			/* When a url does not contain a title, e.g.
			 * index.php?action=edit, then MediaWiki uses the Main
			 * Page title. */
			if (!$effTitle) $effTitle = $fauxParams['title'] ?
			  $fauxParams['title'] : wfMessage('mainpage')->plain();
		}
		$tEffective = Title::newFromText($effTitle);
		$effWebRequest = new FauxRequestEx($fauxParams, $effFullUrl);
		$state = new TVstate(array(
			'tEffective' => $tEffective,
			'tRedirectInfo' => null, # unknown: follow on-demand
			'effectiveWebRequest' => $effWebRequest,
			'effectiveQueryStr' => $effQueryStr,
			'effectiveAction' => $effWebRequest->getVal('action')));
		$hierarchy = new Hierarchy($state);
		$skipStateRebuild = $wgRequest->getBool('skiplinkstaterebuild');
		wfDebug(__METHOD__.": retrieved from request :-\n".
		  "*effective title     : ".($tEffective
		    ? ('"'.$tEffective->getPrefixedText().'"')
		    : '[could not build Title from "'.$effTitle.'"]')."\n".
		  "*base                : ".($base ? "\"$base\"" : 'n/a')."\n".
		  "*expand-node ids     : ".var_export($nodeIds, true)."\n".
		  "*effective full url  : \"$effFullUrl\"\n".
		  "*effective query str : \"$effQueryStr\"\n".
		  '*skiplinkstaterebuild: "'.($skipStateRebuild?'1':'0')."\"\n".
		  "*decoded effective query string as:\n".
		    var_export($fauxParams, true)."\n"
		);

		$op = "<div>\n";
		$htmlArr = $hierarchy->buildHierarchyHtml($nodeIds,
		  $skipStateRebuild, '', true);
		foreach ($htmlArr as $nodeId => $html) {
			if ($html === false) $html = '<input type="hidden" '.
			  "class=\"notfound\">$nodeId</input>";
			$op .= "<div id=\"$nodeId\">$html</div>\n";
		}
		$expandeds = $hierarchy->getExpandedNodesStr();
		$op .= "<form action=\"#\">\n<input type=\"hidden\" ".
		  "id=\"expandednodes\" value=\"$expandeds\" />\n</form>\n";
		$op .= "</div>\n";

		$wgOut->disable();
		header('Content-type: text/xml; charset='.$wgInputEncoding);
//		wfDebug(__METHOD__.": returning as xml:\n$op\n");
		echo $op;

		wfRestoreWarnings();
	}

	/**
	 * Invalidates every treeview-related cache and all articles in the
	 * file cache and client-caches; avoids the parser cache.
	 */
	static function invalidateAll() {
		global $wgMemc;
		wfDebug(__METHOD__.": INVALIDATING the cache of all articles ".
		  "+ the ".'treeview hierarchy + the remappings list'."\n");
		HierarchyPage::invalidateAll_File_Client();
		$key = Hierarchy::getHierarchyKey();
		$wgMemc->delete($key);
		wfLogTreeview('Deleted the hierarchy key ('.$key.
		  ') from the cache.'."\n");
		$key = Hierarchy::getRemappingsKey();
		$wgMemc->delete($key);
		wfLogTreeview('Deleted the remappings key ('.$key.
		  ') from the cache.'."\n");
	}

	/**
	 * Invalidates all articles in the file cache and client-caches;
	 * avoids the parser cache.
	 */
	static function invalidateAll_File_Client() {
		global $wgCacheEpochTouchFile, $wgUseFileCache, $wgCachePages;
		if (!@touch($wgCacheEpochTouchFile)) {
			wfLogTreeview('Touched the cache epoch file ('.
			  $wgCacheEpochTouchFile.').'."\n");
			if ($wgUseFileCache || $wgCachePages) {
				global $wgOut;
				$wgOut->showFatalError(wfMessage(
				  'tv_untouchable', __METHOD__)->escaped());
			}
		}
	}

	/**#@- */ # end public section
}

function wfHierarchy_RenderTreeviewTag($text, $params = array()) {
	global $wgHierarchy, $wgArticle, $wgTitle,
	  $wgSpecialHierarchySavedHookData;

	$wgSpecialHierarchySavedHookData['treeviewtagran'] = true;
	if (!isset($wgHierarchy)) $wgHierarchy = new Hierarchy();
	$pruneids = isset($params['pruneids']) ?
	  explode(',', $params['pruneids']) : array();
	if (isset($params['showtitle'])) {
		$t = Title::newFromText($params['showtitle']);
		$showtitle_parsed = $t->getPrefixedText();
	} else if (isset($params['showthis'])) {
		$t = ($wgArticle ? $wgArticle->getTitle() : $wgTitle);
		$showtitle_parsed = $t->getPrefixedText();
	} else	$showtitle_parsed = null;

	return $wgHierarchy->renderTreeviewTag($showtitle_parsed, $pruneids);
}

/** Encapsulate the hierarchy extension for use as a special page; register the
 * <treeview> tag. */
function wfSpecialHierarchy()
{
	global $wgCacheEpoch, $wgCacheEpochTouchFile, $wgUseFileCache,
	  $wgCachePages, $wgContLanguageCode, $wgParser, $wgSpecialPages;

	$tmp = @filemtime($wgCacheEpochTouchFile);
	if ($tmp !== false) {
		$lastTouched = gmdate('YmdHis', $tmp);
		$org = $wgCacheEpoch;
		$wgCacheEpoch = max($wgCacheEpoch, $lastTouched);
		if ($wgCacheEpoch != $org) {
			wfDebug(__METHOD__.": SET \$wgCacheEpoch to ".
			  "$wgCacheEpoch; was $org\n");
		}
	} else if (($wgUseFileCache || $wgCachePages) &&
	  !@touch($wgCacheEpochTouchFile)) {
		global $wgOut;
		$wgOut->showFatalError(wfMessage('tv_inaccessibletouch',
		  __METHOD__, '<code>'.$wgCacheEpochTouchFile.'</code>')
		  ->escaped());
	}

	$wgParser->setHook('treeview', 'wfHierarchy_RenderTreeviewTag');

	$wgSpecialPages['Hierarchy'] = new HierarchyPage;
}

?>
